----------------------------------------------------------------
-- KiwiPlates Initialization and general stuff
----------------------------------------------------------------

local addonName = 'KiwiPlates'

local addon = {}
LibStub('AceEvent-3.0'):Embed(addon)
_G[addonName] = addon

----------------------------------------------------------------
-- Variables
----------------------------------------------------------------

addon.defaults = setmetatable( {},{ __index = function(t,k) local v={}; t[k]=v; return v end } )

addon.defaults.db = { 
	general = {	
		themes = {
			TANK    = {  default = 'Squared.Tank' },
			DAMAGER = {  default = 'Squared' },
			HEALER  = {  default = 'Squared' },
		}
	},
	themes  = {},
	skins   = {},
	auras   = {},
	minimapIcon = {},
}

addon.statuses = {}
addon.widgets  = {}
addon.skins    = {}
addon.themes   = {}

addon.backup = { skins  = {}, themes = {} }

addon.theme = {}

----------------------------------------------------------------
-- Registering media stuff early
----------------------------------------------------------------
do
	local media = LibStub("LibSharedMedia-3.0", true)
	media:Register('font', 'Yanone Kaffesatz Bold', "Interface\\Addons\\KiwiPlates\\media\\yanone.ttf" )
	media:Register('font', 'FrancoisOne', "Interface\\Addons\\KiwiPlates\\media\\francois.ttf" )
	media:Register("statusbar", "Gradient", "Interface\\Addons\\KiwiPlates\\media\\gradient")
	media:Register("statusbar", "Blizzard Solid White", "Interface\\Buttons\\white8x8")
end

----------------------------------------------------------------
-- Misc functions
----------------------------------------------------------------

function addon:IsSkinEnabled(skin)
	local name = type(skin)=='table' and skin.name or skin
	for _,rule in next, addon.theme.Skins do
		if name == rule[1] then
			return true
		end
	end
	return false
end

----------------------------------------------------------------
-- statuses
----------------------------------------------------------------
function addon:newStatus( name, embed )
	local status = { name = name }
	self.statuses[name] = status
	if embed then
		for i=1,#embed do
			LibStub(embed[i]):Embed(status)	
		end
	end
	return status
end

----------------------------------------------------------------
-- widgets & modules
----------------------------------------------------------------
local widgetProto = {}
widgetProto.__index   = widgetProto
addon.widgetPrototype = widgetProto

function widgetProto:Enable()
	if not self.enabled then
		if self.GetStatuses then 
			self:SetupStatuses(	self:GetStatuses(addon, addon.theme)  )
		end
		if self.statuses then
			for name,status in next,self.statuses do
				if not status.enabled then
					status.enabled = true
					status:OnEnable(addon)
				end	
			end
		end	
		self.enabled = true
		if self.OnEnable then 
			self:OnEnable(addon, addon.theme) 
		end
	end
end

function widgetProto:Disable()
	if self.enabled then
		if self.OnDisable then 
			self:OnDisable(addon) 
		end
		self.enabled = false
	end
end

function widgetProto:SetupStatuses(statuses)
	wipe(self.statuses)
	if statuses then
		for _,name in next, statuses do
			self.statuses[name] = addon.statuses[name]
		end
	end	
end

function widgetProto:SetShown(kPlate, visible)
	local region = kPlate[self.name]
	if region and region.SetShown then
		region:SetShown(visible)
	end	
end

function widgetProto:OnThemeChange(theme)
	addon.ResetAllPlateColors()	
	if self.enabled then self:Disable()  end
	if theme[self.name]	then self:Enable() end
	addon.UpdateStatuses()
	addon.ReSkinPlates(true)
end

function widgetProto:OnSkinChange(skin)
	local key, yek  = self.name, '_'..self.name
	local db = skin[key]
	local enabled = not not db
	if enabled and not self.enabled and addon:IsSkinEnabled(skin) then
		self:Enable()
		addon.UpdateStatuses()
	end
	if self.Layout then
		self:Layout(skin)
	else	
		for _,kPlate in next, addon.kaPlates do
			if kPlate.skinName == skin.name then
				if enabled then
					if not kPlate[key] then
						kPlate[key], kPlate[yek] = kPlate[yek], nil
						self:SetShown(kPlate,true) 
					end
					addon.Notify.PlateCreate(kPlate)
					if kPlate.bPlate then 
						addon.Notify.PlateShow(kPlate, kPlate.bPlate) 
					end
				elseif kPlate[key] then
					self:SetShown(kPlate,false) 			
					kPlate[key], kPlate[yek] = nil, kPlate[key] 
				end
			end	
		end
	end	
end

function widgetProto:GetColorReceivers(skin, receivers)
	local receivers = receivers or {}
	if skin[self.name].acceptColors then
		receivers[self.name] = self.name
	end	
	return receivers
end

function addon:newWidget( name, statuses, embed )
	local widget = setmetatable( {}, widgetProto )
	self.widgets[name] = widget
	widget.enabled = false	
	widget.type = type
	widget.name = name
	widget.statuses = {}
	if embed then
		for i=1,#embed do
			LibStub(embed[i]):Embed(widget)	
		end
	end
	widget:SetupStatuses(statuses)
	return widget
end

----------------------------------------------------------------
-- Options
----------------------------------------------------------------
function addon:Initialize()
	--
	addon.db = addon:GetDB('KiwiPlatesDB', addon.defaults.db)
	--
	local optionsFrame = CreateFrame( "Frame", nil, UIParent )
	optionsFrame.name = "KiwiPlates"
	local button = CreateFrame("BUTTON", nil, optionsFrame, "UIPanelButtonTemplate")
	button:SetText("Open KiwiPlates Options")
	button:SetSize(200,32)
	button:SetPoint('TOPLEFT', optionsFrame, 'TOPLEFT', 20, -20)
	button:SetScript("OnClick", function(self) 
		HideUIPanel(InterfaceOptionsFrame) 
		HideUIPanel(GameMenuFrame) 
		addon.OnChatCommand()
	end)
	InterfaceOptions_AddCategory(optionsFrame)
	self.optionsFrame = optionsFrame
	self.Initialize = nil
end

function addon.OnChatCommand(input)
	if not IsAddOnLoaded("KiwiPlates_Options") then
		if InCombatLockdown() then
			print("KiwiPlates Options cannot be loaded in combat.")
			return
		end
		LoadAddOn("KiwiPlates_Options")		
	end
	if not KiwiPlatesOptions then
		print("You need KiwiPlates_Options addon enabled to be able to configure KiwiPlates.")
		return
	end
	KiwiPlatesOptions:Initialize()
	addon.OnChatCommand = function(input) KiwiPlatesOptions:OnChatCommand(input) end	
	SlashCmdList[ addonName:upper() ] = addon.OnChatCommand
	KiwiPlatesOptions:OnChatCommand(input) 
end

SlashCmdList[ addonName:upper() ] = addon.OnChatCommand
_G[ 'SLASH_'..addonName:upper()..'1' ] = '/kwp'

----------------------------------------------------------------
-- Retrieve database
----------------------------------------------------------------
function addon:GetDB(name, defaults)
	local db = _G[name]
	if not db then
		db = defaults or {}
		_G[name] = db
	end
	self.GetDB = nil
	return db
end

----------------------------------------------------------------
-- Config load
----------------------------------------------------------------
function addon:LoadConfig()
	for _,type in pairs( {'themes', 'skins'} ) do
		for name, data in pairs(self.db[type]) do
			addon[type][name] = data
		end
	end
	self.LoadConfig = nil
end

----------------------------------------------------------------
-- Init
----------------------------------------------------------------
local frame = CreateFrame('Frame')
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PLAYER_LOGIN")
frame:SetScript("OnEvent", function(frame, event, name)
	if event == "ADDON_LOADED" and name == addonName then
		addon:Initialize()		
	end
	if (not addon.Initialize) and IsLoggedIn() then
		frame:UnregisterAllEvents()
		frame:SetScript("OnEvent", nil)
		addon:LoadConfig()
		addon:InitTheme()
		addon:PlatesRun(frame)
	end	
end)
