local media = LibStub("LibSharedMedia-3.0", true)

local addon = KiwiPlates
local widget = addon:newWidget( 'CastBar' )
local Notify = addon.Notify
local PlateSendToBack   = addon.PlateSendToBack
local PlateBringToFront = addon.PlateBringToFront

local unpack = unpack

local defaultColor = addon.defaults.color.white
local defaultCoord = {0,1,0,1}
local opaque, uColor, iColor 

local function Layout(kPlate)
	local CastBar    = kPlate.CastBar
	local db         = kPlate.db.CastBar
	local barBSize   = (db.borderSize or 0) * 2	
	local x          = db.left
	local y          = db.top
	local width      = db.width  or kPlate.width
	local height     = db.height or kPlate.height
	local barWidth   = width  - barBSize
	local barHeight  = height - barBSize
	
	CastBar.kPlate = kPlate
	CastBar:SetFrameLevel( kPlate:GetFrameLevel() )
	CastBar:SetStatusBarTexture( media:Fetch('statusbar', db.barTexture) )
	CastBar:ClearAllPoints()
	CastBar:SetPoint('LEFT',kPlate, 'CENTER', x-barWidth/2, y)
	CastBar:SetSize( barWidth, barHeight )
	CastBar.barWidth = barWidth

	local kBorder = CastBar.kBorder
	if kBorder then
		kBorder:SetDrawLayer( 'BACKGROUND', 0)
		kBorder:ClearAllPoints()
		kBorder:SetPoint('CENTER')
		kBorder:SetSize( width, height )
		kBorder:SetTexture( unpack(db.borderColor) )
	end
		
	local kBackground = CastBar.kBackground
	if kBackground then
		kBackground:SetDrawLayer( 'BACKGROUND', 1)
		kBackground:ClearAllPoints()
		kBackground:SetPoint('CENTER')
		kBackground:SetSize( barWidth, barHeight )
		kBackground:SetTexture( unpack(db.backColor) )
	end

	local kTex = CastBar.kTexture
	if kTex then
		local dbx = db.Texture
		kTex:SetDrawLayer( dbx.layer or 'BACKGROUND', dbx.sublayer or 0)
		kTex:SetTexture( dbx.filename )
		kTex.defaultColor = dbx.color or defaultColor
		kTex:SetVertexColor( unpack(kTex.defaultColor) )
		kTex:SetPoint('CENTER', dbx.left or 0, dbx.top or 0 )
		kTex:SetTexCoord( unpack(db.coord or defaultCoord) )
		kTex:SetSize( dbx.width or kPlate.width, dbx.height or kPlate.height )
	end

	local kSpellIcon = CastBar.kSpellIcon
	if kSpellIcon then
		local dbx = db.SpellIcon
		local size2 = (dbx.borderSize or 0) * 2
		kSpellIcon:SetDrawLayer( 'ARTWORK', 5)
		kSpellIcon:ClearAllPoints()
		kSpellIcon:SetPoint('CENTER', dbx.left, dbx.top)
		kSpellIcon:SetSize( dbx.width, dbx.height )
		kSpellIcon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
		kSpellIcon.borderColor = dbx.borderColor or defaultColor
		local kSpellIconBg = CastBar.kSpellIconBg
		kSpellIconBg:SetDrawLayer( dbx.layer or 'ARTWORK', dbx.sublayer or 4)
		kSpellIconBg:ClearAllPoints()
		kSpellIconBg:SetTexture( dbx.filename or unpack(defaultColor) )
		kSpellIconBg:SetVertexColor( unpack(kSpellIcon.borderColor) )
		kSpellIconBg:SetPoint('CENTER', dbx.left, dbx.top)
		kSpellIconBg:SetSize( dbx.width+size2, dbx.height+size2 )
	end

	local kSpellText = CastBar.kSpellText
	if kSpellText then
		local dbx = db.SpellText
		kSpellText:SetFont( media:Fetch('font', dbx.fontName), dbx.fontSize, dbx.fontFlags)
		kSpellText:SetTextColor( unpack(dbx.fontColor) )
		kSpellText:SetSize( width+dbx.AdjustX, height+dbx.AdjustY )		
		kSpellText:SetJustifyH( dbx.JustifyH )
		kSpellText:SetJustifyV( dbx.JustifyV )	
		kSpellText:ClearAllPoints()
		kSpellText:SetPoint('CENTER')	
	end
	
	local kTimeText = CastBar.kTimeText
	if kTimeText then
		local dbx = db.TimeText
		kTimeText:SetFont( media:Fetch('font', dbx.fontName), dbx.fontSize, dbx.fontFlags)
		kTimeText:SetTextColor( unpack(dbx.fontColor) )		
		kTimeText:SetSize( width+dbx.AdjustX, height+dbx.AdjustY )
		kTimeText:SetJustifyH( dbx.JustifyH )
		kTimeText:SetJustifyV( dbx.JustifyV )	
		kTimeText:ClearAllPoints()
		kTimeText:SetPoint('CENTER')
	end	
end

local function Create(kPlate)
	local db = kPlate.db.CastBar
	local CastBar = kPlate.CastBar or CreateFrame("StatusBar", nil, kPlate)
	if db.SpellText then
		CastBar.kSpellText = CastBar.kSpellText or CastBar:CreateFontString(nil, 'ARTWORK')
	end	
	if db.TimeText then
		CastBar.kTimeText = CastBar.kTimeText or CastBar:CreateFontString(nil, 'ARTWORK')
	end	
	if db.SpellIcon and not db.SpellIcon.disabled then
		CastBar.kSpellIcon   = CastBar.kSpellIcon or CastBar:CreateTexture()
		CastBar.kSpellIconBg = CastBar.kSpellIconBg or CastBar:CreateTexture()			
	end
	if db.backColor then
		CastBar.kBackground = CastBar.kBackground or CastBar:CreateTexture()
		CastBar.kBorder     = CastBar.kBorder or CastBar:CreateTexture()
	end	
	if db.Texture then
		CastBar.kTexture = CastBar.kTexture or CastBar:CreateTexture()
	end
	kPlate.CastBar = CastBar
	Layout(kPlate)
	return CastBar
end
		
local function bBarShow(bCastBar)
	local bPlate = bCastBar.bPlate
	if bCastBar:IsVisible() and bPlate:IsVisible() then
		local kPlate = bPlate.kPlate
		if kPlate then
			if kPlate.db.CastBar and addon.theme.CastBar then
				local CastBar = kPlate.CastBar
				if CastBar then
					CastBar:Show()
				else
					CastBar = Create(kPlate)
				end
				local min,max = bCastBar:GetMinMaxValues()
				CastBar:SetMinMaxValues( min, max )
				bCastBar.CastBar = CastBar
				local kSpellText = CastBar.kSpellText
				if kSpellText then
					kSpellText:SetText( bPlate.bSpellText:GetText() )
				end	
				local kSpellIcon = CastBar.kSpellIcon
				if kSpellIcon then
					kSpellIcon:SetTexture( bPlate.bSpellIcon:GetTexture() )
					if bPlate.bShield:IsShown() then
						CastBar.kSpellIconBg:SetVertexColor( uColor[1], uColor[2], uColor[3], uColor[4] )
						CastBar:SetStatusBarColor( uColor[1], uColor[2], uColor[3] )
					else
						CastBar.kSpellIconBg:SetVertexColor( unpack(kSpellIcon.borderColor) )
						CastBar:SetStatusBarColor( iColor[1], iColor[2], iColor[3] )
					end
				else
					if bPlate.bShield:IsShown() then
						CastBar:SetStatusBarColor( uColor[1], uColor[2], uColor[3] )
					else
						CastBar:SetStatusBarColor( iColor[1], iColor[2], iColor[3] )
					end
				end	
				if opaque then
					PlateBringToFront( kPlate, bPlate, 'CastBar' )
				end
			end	
		end	
	end	
end

local function bBarHide(bCastBar)
	local CastBar = bCastBar.CastBar
	if CastBar then
		bCastBar.CastBar = nil	
		CastBar:Hide() 
		if opaque then
			PlateSendToBack( CastBar.kPlate, bCastBar.bPlate, 'CastBar' )
		end	
	end
end

local function bSetValue(bCastBar, value)
	local CastBar = bCastBar.CastBar
	if CastBar then
		if value>0 then
			CastBar:SetValue( value )
			CastBar.kTimeText:SetFormattedText( '%.1f', value )
		else
			bBarHide(bCastBar) 
		end
	end	
end

local function bPlateCreate(bPlate)
	local bCastBar = bPlate.bCastBar
	if not bCastBar.bPlate then
		bCastBar.bPlate = bPlate		
		bCastBar:HookScript('OnShow', bBarShow)
		bCastBar:HookScript('OnHide', bBarHide)
		bCastBar:HookScript('OnValueChanged', bSetValue )
	end	
end

local function PlateShow(kPlate,bPlate)
	local bCastBar = bPlate.bCastBar
	if bCastBar:IsVisible() then
		bBarShow(bCastBar)
	end
end

local function PlateHide(kPlate)
	local CastBar = kPlate.CastBar
	if CastBar then 
		kPlate.bPlate.bCastBar.CastBar = nil
		CastBar:Hide() 
	end
end

function widget:Layout(skin)
	for _,kPlate in next, addon.kaPlates do
		if kPlate.CastBar and skin.name == kPlate.skinName then
			Layout(kPlate)
		end
	end	
end

function widget:OnEnable(addon, theme)
	local themedb = theme.CastBar
	if themedb then
		opaque = themedb.opaquePlates
		uColor = themedb.uninterruptibleColor or addon.defaults.color.red
		iColor = themedb.interruptibleColor   or addon.defaults.color.white
		addon.RegisterNotify( 'bPlateCreate', bPlateCreate )
		addon.RegisterNotify( 'PlateShow', PlateShow )
		addon.RegisterNotify( 'PlateHide', PlateHide )
		for _,bPlate in next,addon.bPlates do bPlateCreate( bPlate ) end	
	end	
end

function widget:OnDisable(addon)
	addon.UnregisterNotify( 'bPlateCreate', bPlateCreate )	
	addon.UnregisterNotify( 'PlateShow', PlateShow )		
	addon.UnregisterNotify( 'PlateHide', PlateHide )	
end
