local addon   = KiwiPlates
local Options = KiwiPlatesOptions

-- function Options:MakeColorReceiversOptions(options)
do
	local receivers = {}
	local header = { type = 'header', order = 100, name = 'Color Receivers' }
	local colorReceivers = {
		type = 'multiselect',
		name = 'Widgets to colorize',
		order = 110,
		inline = true,
		values = function(info)
			wipe(receivers)
			for _, rule in next,info.handler.theme.Skins do
				local skin = addon.skins[ rule[1] ]
				if skin then
					for key in next,skin do
						local widget = addon.widgets[key]
						if widget then
							widget:GetColorReceivers(skin, receivers)
						end
					end
				end	
			end
			return receivers
		end
	}
	local colorPriority = {
		type = 'select',
		name = 'Color priority',
		width = 'half',
		order = 120,
		values = { 1,2,3,4,5,6,7,8,9 }
	}
	function Options:MakeColorReceiversOptions(options)
		options = options or {}
		options.headerReceivers = header
		options.colorReceivers = colorReceivers
		options.colorPriority  = colorPriority
		return options
	end
end

-- Options:MakeColorsGroupOptions(order, name)
function Options:MakeColorsGroupOptions(order, name, colors, options)
	options = options or {}
	if name then options.colorsHeader = { type = 'header', order = order, name = name }	end
	options.colors = { type = 'group', order = order + 0.5 , inline = true, args = {} }
	for order,key in next, colors do
		options.colors.args[key] = {
			type = 'color',
			order = order,
			name = key,
			hasAlpha = true,
			width = "normal",
		}
	end	
	return options
end

-- Options:MakeMultiselectOptions(order, name, values)
function Options:MakeMultiselectOptions(order, name, values)
	local list = {}
	for k,v in next,values do
		list[k] = k
	end
	return { type = 'multiselect', order= order, name = name, values = list }
end

-- Options:MakeSizeOptions(order, name)
do
	local args = {
		width = {
			type = 'range',
			name = 'Width',
			order = 3,
			step = 1,
			min = 1,
			softMin = 25,
			softMax = 220
		},
		height = {
			type = 'range',
			name = 'Height',
			order = 4,
			step = 1,
			min = 1,
			softMin = 5,
			softMax = 220
		},	
	}
	function Options:MakeSizeOptions(order, name)
		return { name = name or 'Size', type = 'group', inline = true, order = order, args = args } 
	end
end

-- Options:MakePosSizeOptions(order, name)
do
	local args = {
		left = {
			type = 'range',
			name = 'Left',
			order = 1,
			step = 1,
			softMin = -100,
			softMax = 100,
		},
		top = {
			type = 'range',
			name = 'Top',
			order = 2,
			step = 1,
			softMin = -100,
			softMax = 100,
		},
		width = {
			type = 'range',
			name = 'Width',
			order = 3,
			step = 1,
			min = 0,
			softMin = 0,
			softMax = 200
		},
		height = {
			type = 'range',
			name = 'Height',
			order = 4,
			step = 1,
			min = 0,
			softMin = 0,
			softMax = 200
		},	
	}
	function Options:MakePosSizeOptions(order, name)
		return { name = name or 'Pos&Size', type = 'group', inline = true, order = order, args = args } 
	end
end

-- Options:MakePosOptions(order, name)
do
	local args = {
		left = {
			type = 'range',
			name = 'Left',
			order = 1,
			step = 1,
			softMin = -100,
			softMax = 100,
		},
		top = {
			type = 'range',
			name = 'Top',
			order = 2,
			step = 1,
			softMin = -100,
			softMax = 100,
		},
	}
	function Options:MakePosOptions(order, name)
		return { name = name or 'Position', type = 'group', inline = true, order = order, args = args } 
	end
end


-- Options:MakeSolidDecorationOptions(order, name)
do
	local args = {
		backColor = { 
			order = 1,		
			type = 'color',
			width = 'half',
			name = 'Back',
			desc = 'Background Color',
		},
		borderColor = {
			order = 2,
			type = 'color',
			width = 'half',			
			name = 'Border',
			desc = 'Border Color',
		},	
		borderSize = {
			order = 3,
			type = 'range',
			name = 'Border Size',
			step = 1,
			min = 1,
			softMin = 1,
			softMax = 220
		},
	}
	function Options:MakeSolidDecorationOptions(order, name)
		return { name = name or 'Decoration', type = 'group', inline = true, order = order, args = args } 
	end
end

-- Options:MakeSolidBorderOptions(order, name)
do
	local args = {
		borderColor = {
			order = 1,
			type = 'color',
			width = 'half',			
			name = 'Border',
			desc = 'Border Color',
		},	
		borderSize = {
			order = 2,
			type = 'range',
			name = 'Border Size',
			step = 1,
			min = 1,
			softMin = 1,
			softMax = 220
		},
	}
	function Options:MakeSolidBorderOptions(order, name)
		return { name = name or 'Border', type = 'group', inline = true, order = order, args = args } 
	end
end

-- Options:MakeFontOptions(order, name)
-- Options:MakeFontColorOptions(order, name)
do
	local fontFlagsValues = { 
		["NONE"] = "Soft",
		["OUTLINE"] = "Soft Thin", 
		["THICKOUTLINE"] = "Soft Thick", 
		["MONOCHROME"] = "Sharp",
		["MONOCHROME, OUTLINE"] = "Sharp Thin", 
		["MONOCHROME, THICKOUTLINE"] = "Sharp Thick", 
	}
	local font = {
		fontName = {
			type = "select", dialogControl = "LSM30_Font",
			order = 1,
			name = "Name",
			values = AceGUIWidgetLSMlists.font,
		},
		fontSize = {
			type = "range",
			order = 2,
			name = 'Size',
			min = 6,
			max = 24,
			step = 1,
		},
		fontFlags = {
			type = "select",
			order = 3,
			name = "Border",
			values = fontFlagsValues,
		}
	}
	local fontColor = {
		fontName = {
			type = "select", dialogControl = "LSM30_Font",
			order = 1,
			name = "Name",
			values = AceGUIWidgetLSMlists.font,
		},
		fontFlags = {
			type = "select",
			order = 2,
			name = "Border",
			values = fontFlagsValues,
		},
		fontSize = {
			type = "range",
			order = 3,
			name = 'Size',
			min = 6,
			max = 24,
			step = 1,
		},
		fontColor = {
			type = 'color',
			name = 'Color',
			desc = 'Font Color',
			order = 4
		}
	}
	function Options:MakeFontOptions(order, name)
		return { name = name or 'Font', type = 'group', inline = true, order = order, args = font } 
	end
	function Options:MakeFontColorOptions(order, name)
		return { name = name or 'Font', type = 'group', inline = true, order = order, args = fontColor } 
	end
end

-- Options:MakeTextJustifyOptions(order, name)
do
	local args = {
		JustifyH = {
			type = "select",
			order = 1,
			name = "Horizontal",
			values = { CENTER = 'CENTER', LEFT = 'LEFT', RIGHT = 'RIGHT', },
		},
		JustifyV = {
			type = "select",
			order = 2,
			name = "Vertical",
			values = { MIDDLE = 'CENTER', TOP = 'TOP', BOTTOM = 'BOTTOM', },
		},
	}
	function Options:MakeTextJustifyOptions(order, name)
		return { name = name or 'Justify', type = 'group', inline = true, order = order, args = args }
	end
end		

-- Options:MakeTextJustifyAdjustOptions(order, name)
do
	local args = {
		JustifyH = {
			type = "select",
			order = 1,
			name = "Justify Horizontal",
			values = { CENTER = 'CENTER', LEFT = 'LEFT', RIGHT = 'RIGHT', },
		},
		AdjustX = {
			type = "range",
			order = 2,
			step = 1,
			softMin = -32,
			softMax = 32,
			name = "X Adjust",
		},
		JustifyV = {
			type = "select",
			order = 3,
			name = "Justify Vertical",
			values = { MIDDLE = 'CENTER', TOP = 'TOP', BOTTOM = 'BOTTOM', },
		},
		AdjustY = {
			type = "range",
			order = 4,
			step = 1,
			softMin = -32,
			softMax = 32,
			name = "Y Adjust",
		},
	}
	function Options:MakeTextJustifyAdjustOptions(order, name)
		return { name = name or 'Placement', type = 'group', inline = true, order = order, args = args }
	end
end		

	
-- Options:MakeStatusBarOptions(order, name)	
do
	local args = {
		barTexture = {
			type = 'select',
			name = 'Texture',
			desc = 'The texture used for the bar.',
			dialogControl = 'LSM30_Statusbar',
			values = AceGUIWidgetLSMlists.statusbar,
		},	
	}
	function Options:MakeStatusBarOptions(order, name)
		return { name = name or 'Bar', type = 'group', inline = true, order = order, args = args }
	end	
end

-- Options:MakeAuraSizeOptions(order, name)
do
	local args = {
		auraWidth = {
			type = 'range',
			name = 'Width',
			order = 1,
			step = 1,
			min = 1,
			softMin = 5,
			softMax = 100,
		},
		auraHeight = {
			type = 'range',
			name = 'Height',
			order = 2,
			step = 1,
			min = 1,
			softMin = 5,
			softMax = 100,
		},	
		auraSpacing = {
			type = 'range',
			name = 'Spacing',
			order = 3,
			step = 1,
			min = 1,
			softMin = 5,
			softMax = 100,
		},
	}
	function Options:MakeAuraSizeOptions(order, name)
		return { name = name or 'Aura', type = 'group', inline = true, order = order, args = args } 
	end
end

-- Options:MakeMultiSelectOptions(order, name, keys)
function Options:MakeMultiSelectOptions(order, name, keys)
	return { name = name or '', type = 'multiselect', inline = true, order = order, values = keys } 
end

-- function Options:MakeDrawLayerOptions(order, name)
do
	local layers = {
		layer = {
			type = "select",
			order = 1,
			name = "Layer",
			values = { BACKGROUND = 'BACKGROUND', BORDER = 'BORDER', ARTWORK = 'ARTWORK', HIGHLIGHT = 'HIGHLIGHT', OVERLAY = 'OVERLAY',	},
		},
		sublayer = {
			type = "range",
			order = 2,
			name = 'Sublayer',
			min = -8,
			max = 7,
			step = 1,
		},
	}
	function Options:MakeDrawLayerOptions(order, name)
		return { name = name or 'Layer', type = 'group', inline = true, order = order, args = layers } 
	end
end

function Options:MakeHealthTextFormatOptions(order, name, db)
	return { type = "select", order = order or 3, name = 'textFormat', values = { 'Percent', 'Health', 'Health|Percent', 'Percent|Health' } }, 'textFormat'
end

function Options:MakeTextureFilenameOptions(order, name, db)
	if db['__filename'] then
		return { type = "select", order = order, name = 'texture', values = addon:GetWidgetTexturesValues( db['__filename'] ) }, 'filename'
	end	
end

-- function Options:MakeAutomaticOptions(db)
do	
	local rules = {
		 { 'MakePosSizeOptions', 'left', 'top', 'width', 'height' }, 
		 { 'MakeSizeOptions', 'width', 'height' },
		 { 'MakePosOptions', 'left', 'top' },
		 { 'MakeAuraSizeOptions', 'auraWidth', 'auraHeight', 'auraSpacing' },		 
		 { 'MakeSolidDecorationOptions', 'backColor', 'borderColor', 'borderSize' },
		 { 'MakeSolidBorderOptions',  'borderColor', 'borderSize' },
		 { 'MakeFontColorOptions' , 'fontName', 'fontFlags', 'fontSize', 'fontColor' },
		 { 'MakeFontOptions', 'fontName', 'fontSize', 'fontFlags'  },
		 { 'MakeTextJustifyAdjustOptions' , 'JustifyH', 'JustifyV', 'AdjustX', 'AdjustY' },
		 { 'MakeTextJustifyOptions' , 'JustifyH', 'JustifyV' },
		 { 'MakeStatusBarOptions' , 'barTexture' },
		 { 'MakeDrawLayerOptions',  'layer', 'sublayer' }, -- TODO: This is currently blacklisted, i.e. never displayed, review
		 { 'MakeHealthTextFormatOptions' , 'textFormat' },
		 { 'MakeTextureFilenameOptions' , 'filename' },
	}
	local blacklist = { Textures = true, Texture = true, texture = true, layer= true, sublayer= true }
	function Options:MakeAutomaticOptions(db, name, options)
		options = options or {}
		local options_single, unused, order = {}, {}, 1
		local function get_key()
			order = order + 1
			return 'K' .. order
		end
		local function is_first_char_upper(s)
			s = strsub(s,1,1)
			return s == s:upper()
		end
		local function is_color_table(value)
			return type(value) == 'table' and #value>=3 and #value<=4
		end
		local function is_bool_table(k,t)
			if type(t)=='table' and (not is_first_char_upper(k)) then
				for _,v in next,t do
					if type(v)~='boolean' then return end	
				end
				return true
			end	
		end
		local function process_rule(rule)
			local c = 1
			for i=2,#rule do
				if unused[rule[i]] then c = c + 1 end 
			end
			if c == #rule then
				local opt, key = self[rule[1]](self, order, nil, db)
				if opt then
					if key then
						options_single[key] = opt
					else
						options[get_key()] = opt
					end
				end	
				for i=2,#rule do
					unused[rule[i]] = nil
				end
			end
		end
		for k in next, db do
			if strsub(k,1,1) ~= '_' and not (db.__HIDDEN_FIELDS and db.__HIDDEN_FIELDS[k]) and not blacklist[k] then
				unused[k] = true
			end	
		end
		for _,rule in ipairs(rules) do
			process_rule(rule)
		end
		if next(unused) then
			for key in next,unused do
				local value = db[key]
				if type(value) == 'number' and db['__'..key] then
					options_single[key] = { type = "select", order = 3, name = key, values = db['__'..key] }
					unused[key] = nil					
				elseif type(value) == 'number' then
					options_single[key] = { type = "range", order = 0,	name = key, softMin = 0, softMax = 100,	step = 1 }
					unused[key] = nil
				elseif type(value) == 'boolean' then
					options_single[key] = { type = "toggle", order = 1, name = key }
					unused[key] = nil
				elseif type(value) == 'string' then
					options_single[key] = { type = "input", order = 4,	width= 'full',	name = key }
					unused[key] = nil
				elseif is_color_table(value) then
					options_single[key] =  { type = 'color', order = 2, name = key, hasAlpha = (#value==4) }
					unused[key] = nil
				elseif is_bool_table(key, value) then	
					options[key] =  self:MakeMultiselectOptions(0.75, key, value)
					unused[key] = nil
				end	
			end
		end	
		if next(options_single) then
			options[ get_key() ] = { name = name, type = 'group', inline = true, order = 0.5, args = options_single } 
		end
		for key in next,unused do
			local value = db[key]
			if type(value) == 'table' and #value==0 and is_first_char_upper(key) then
				options[key] = { name = key, type = 'group', inline = false, order = order, args = self:MakeAutomaticOptions(value, key) }
				unused[key] = nil
				order = order + 1
			end	
		end
		return options
	end
end

