----------------------------------------------------------------
--
----------------------------------------------------------------

local addon   = KiwiPlates
local Options = KiwiPlatesOptions

local theme
local modules = {}
local options = {}

----------------------------------------------------------------
--
----------------------------------------------------------------

function Options:GetSelectedTheme()
	return theme
end

----------------------------------------------------------------
--
----------------------------------------------------------------

local optionEnabledModules = {
	type = 'multiselect',
	name = 'Enabled Modules',
	order = 10,
	get = function(info, key) return type(modules[key]) == 'table' end,
	set = function(info, key, value)
		theme[key]   = value and Options:GetModuleDefaults(key, true)        or nil
		modules[key] = value and Options:MakeModuleOptions(key, theme[key] ) or nil
		info.handler:OnChange(info, key)
	end,
	confirm = function(info, key, value)
		return (not value) and 'Are you sure you want to disable: '..key
	end,
	values = function()
		local modules_list = {}
		for name, value in next,Options.module_defaults do
			if not value.__ALWAYS_ENABLED then
				modules_list[name] = name
			end	
		end
		return modules_list
	end	
}

----------------------------------------------------------------
--
----------------------------------------------------------------

local handler = {}
function handler:OnChange(info, modName)
	-- database modified, copy database if necessary
	local name = theme.name
	if not addon.db.themes[name] then
		addon.db.themes[name] = theme
		addon.backup.themes[name] = nil
	end
	if not theme.__MODIFIED then
		theme.__MODIFIED = true
	end
	-- notify change to module
	if theme.name == addon.theme.name then	
		modName = modName or self.dbname
		local module = addon.widgets[modName]
		if module then
			module:OnThemeChange(theme)
		end	
	end
end

----------------------------------------------------------------
--
----------------------------------------------------------------

local function LoadTheme(info, value)
	theme = nil
	options.theme.name = ''
	wipe(modules)
	if value then
		options.theme.name = value
		theme = addon.themes[value]
		info.handler.theme = theme
		modules.enabledModules = optionEnabledModules
		for key,db in next, theme do
			modules[key] = Options:MakeModuleOptions(key, db)
		end
		for key,db in next, Options.module_defaults do
			if db.__ALWAYS_ENABLED and not modules[key] then
				modules[key] = Options:MakeModuleOptions(key)
			end	
		end
	end	
end

----------------------------------------------------------------
--
----------------------------------------------------------------

options.themes = {
	type = "select",
	order = 10,
	width = "normal",
	name = "Select Theme",
	desc = "",
	get = function(info, value)
		if not theme and addon.theme then
			LoadTheme(info, addon.theme.name)
		end
		return theme and theme.name or ''
	end,
	set = LoadTheme,
	values = function()
		local t = {}
		for k,v in pairs(addon.themes) do
			t[k] = k
		end
		return t
	end
}

options.clone = {
	type = "input",
	order = 20,
	width = "normal",
	name = "Clone Theme",
	desc = "Type a name for the new theme and click Accept.",
	get = function() end,
	set = function(info, name) 
		theme = addon.CopyTable(theme)
		theme.__THEME_BASE = theme.__THEME_BASE or theme.name
		theme.name = name
		addon.db.themes[name] = theme
		addon.themes[name]    = theme
		LoadTheme(info, name)
	end,
	hidden   = function() return not theme end,
	validate = function(info, name) return not addon.themes[name] and strlen(name)>3 or "Theme name too short or already in use, type another theme name"  end,
	confirm  = function() return "Are you sure you want to duplicate this theme ?" end,	
}

options.reset = {
	type = "execute",
	order = 30,
	width = "half",
	name = "Reset",
	func = function(info) 
		addon.db.themes[theme.name] = nil
		ReloadUI()
	end,
	hidden = function() return not (theme and theme.__MODIFIED and not theme.__THEME_BASE) end,
	confirm = function() return "Are you sure you want to reset this theme ? UI will be Reloaded" end,
}

options.delete = {
	type = "execute",
	order = 40,
	width = "half",
	name = "Delete",
	func = function(info) 
		addon.themes[theme.name] = nil
		addon.db.themes[theme.name] = nil
		LoadTheme(info, nil)
	end,
	hidden   = function() return not (theme and theme.__THEME_BASE) end,
	disabled = function()
		for _,name in next,addon.db.general.themes do
			if theme.name==name then return true end
		end
	end,
	confirm  = function() return "Are you sure you want to delete this theme ?"  end,
}

options.theme = {
	type = "group",
	name = "",
	args = modules,
}

----------------------------------------------------------------
--
----------------------------------------------------------------

Options:RegisterCategoryOptions( 'Themes', function()
	theme = nil
	wipe(modules)
	return options
end, { order = 20, handler = handler } )
