local PATH        = 'Interface\\addons\\KiwiPlates_Rounded\\'
local defaults    = KiwiPlates.defaults
local FONT        = defaults.fonts.STANDARD
local FONT_SMALL  = defaults.fonts.SMALL
local WHITE       = defaults.color.white
local BLACK       = defaults.color.black
local TRANSPARENT = defaults.color.transparent

KiwiPlates:RegisterTheme( 'Rounded', {
	Plate = {
		alpha = 0.5,
	},
	Skins = { 
		{ 'Rounded.Standard' }
	},
	Target = {
		glowEnabled  = true,
		glowColor    = { 1, 1, 1, 1 },
		colorReceivers = { "GlowBorder" },
		colorPriority = 7,
		opaqueWhenNoTarget = true,
	},
	Mouseover = {
		alpha = 1,
		layer = 'OVERTARGET',
		glowEnabled = true,
		glowColor = { 1, 1, 1, 0.75 },
		colorReceivers = { "GlowBorder" },
		colorPriority = 6,
	},
	Reaction = {
		npcColorize    = "reaction",
		playerColorize = "reaction",
		colors = defaults.colors.Reaction,
		colorReceivers = { 'HealthBar','NameText' },
		colorPriority = 1,		
	},	
	Classification = {
		colors = defaults.colors.Classification,
		colorReceivers = { 'LevelText' },
		colorPriority = 2,
	},	
	Threat = {
		colors = defaults.colors.Threat,
		colorReceivers = { "GlowBorder", "NameText" },
		colorPriority = 9,
	},
} )

KiwiPlates:RegisterSkin( 'Rounded.Standard' , {
	Plate = { 
		disableHighlight = false,
		left = 0, top = 0, width  = 256, height =  32, 
		__HIDDEN_FIELDS = { width = true, height = true },
	},
	Mouseover = {
		disableHighlight = false,
		highlightColor = { 1,1,1,0.2 },
		filename = PATH .. 'standard-high',
	},
	Target = {
		disableHighlight = false,	
		highlightColor = { 1,1,1,0.2 },
		filename  = PATH .. 'standard-high',
	},
	GlowBorder = {
		acceptColors = true,
		layer = "BACKGROUND",
		sublayer = -1,
		color = TRANSPARENT,
		filename = PATH .. 'standard-border',
	},
	ClassificationIcon = {
		applyClassificationColor = true,
		left = -67, top = 0, width = 14, height = 14,
		display = { boss = true, elite = true, rare = true, normal = false, trivial = false, player = false },
		filename = PATH .. 'circle', __filename = 'ClassificationIcon',
	},
	HealthBar = {
		acceptColors = true,
		width = 142, height = 9, 
		barTexture  = 'Gradient',
		Texture = {	layer = "BACKGROUND", sublayer = 7,	filename = PATH .. 'standard-bar' },
		__HIDDEN_FIELDS = { width = true, height = true },
	},
	NameText = {
		acceptColors = true,
		left = 0, top = 14, width = false, height = 20,
		fontName = FONT,
		fontColor = WHITE,
		fontSize =  11,
		fontFlags = 'OUTLINE',
		JustifyH = 'CENTER',
		JustifyV = 'BOTTOM',
	},
	TargetIcon = {
		left = 0, top = 16, width = 22, height = 22,
	},
	CastBar = {
		left = 0, top = -15, width = 140, height = 10,		
		barTexture = 'Gradient',
		Texture = {
			layer = "ARTWORK", sublayer = 7, coords = {0,1,0,1 },
			filename = PATH .. 'standard-bar',
		},	
		SpellIcon = {
			left = -90, top = 8, width = 26, height = 26,
			borderSize = 2,	sublayer = 7,
			filename = PATH .. 'spell',
			__HIDDEN_FIELDS = { borderSize = true },
		},
		SpellText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'LEFT',
			JustifyV = 'MIDDLE',
			AdjustX = 0,
			AdjustY = 0,
		},
		TimeText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'RIGHT',
			JustifyV = 'MIDDLE',
			AdjustX = 0,
			AdjustY = 0,
		},
	},
	Auras = {
		left = -60, top = 28, 
		auraWidth = 22, auraHeight = 16, auraSpacing = 2,
		borderSize = 1,
		layer = "ARTWORK", sublayer = 7,
		filename = PATH .. 'spell',
		TimeText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  11,
			fontFlags = 'OUTLINE',
			JustifyH = 'LEFT',
			JustifyV = 'TOP',
			AdjustX = -1,
			AdjustY = 8,
		},
		CountText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'RIGHT',
			JustifyV = 'BOTTOM',	
			AdjustX = 0,
			AdjustY = 0,
		},
	},
} )

KiwiPlates:RegisterSkin( 'Rounded.Medium' , {
	Plate = { 
		left = 0, top = 0, width  = 256, height =  32, 
		__HIDDEN_FIELDS = { width = true, height = true },
	},
	Mouseover = {
		disableHighlight = false,
		highlightColor = { 1,1,1,0.2 },
		filename = PATH .. 'medium-high',
		__HIDDEN = true,
	},
	Target = {
		disableHighlight = false,
		highlightColor = { 1,1,1,0.2 },
		filename  = PATH .. 'medium-high',
		__HIDDEN = true,
	},
	GlowBorder = {
		acceptColors = true,
		layer = "BACKGROUND",
		sublayer = -1,
		color = TRANSPARENT,
		filename = PATH .. 'medium-border',
	},	
	ClassificationIcon = {
		applyClassificationColor = true,
		left = -57, top = 0, width = 14, height = 14,
		display = { boss = true, elite = true, rare = true, normal = false, trivial = false, player = false },		
		filename = PATH .. 'circle', __filename = 'ClassificationIcon',
	},
	HealthBar = {
		acceptColors = true,
		width = 124, height = 9, barTexture  = 'Gradient',
		Texture = {	layer = "BACKGROUND", sublayer = 7,	color = WHITE, filename = PATH .. 'medium-bar' },
		__HIDDEN_FIELDS = { width = true, height = true },
	},
	NameText = {
		acceptColors = true,
		left = 0, top = 14, width = 150, height = 20,
		fontName = FONT,
		fontColor = WHITE,
		fontSize =  11,
		fontFlags = 'OUTLINE',
		JustifyH = 'CENTER',
		JustifyV = 'BOTTOM',
	},
	TargetIcon = {
		left = 0, top = 16, width = 22, height = 22,
	},
	CastBar = {
		left = 0, top = -15, width = 140, height = 10,		
		barTexture = 'Gradient',
		Texture = {
			layer = "ARTWORK", sublayer = 7, coords = {0,1,0,1 },
			filename = PATH .. 'medium-bar',
		},	
		SpellIcon = {
			left = -90, top = 8, width = 26, height = 26,
			borderSize = 2,	sublayer = 7,
			filename = PATH .. 'spell',
		},
		SpellText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'LEFT',
			JustifyV = 'MIDDLE',
			AdjustX = 0,
			AdjustY = 0,
		},
		TimeText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'RIGHT',
			JustifyV = 'MIDDLE',
			AdjustX = 0,
			AdjustY = 0,
		},
	},
	Auras = {
		left = -52, top = 25, 
		auraWidth = 22, auraHeight = 16, auraSpacing = 2,
		borderSize = 1,
		layer = "ARTWORK", sublayer = 7,
		filename = PATH .. 'spell',
		TimeText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  11,
			fontFlags = 'OUTLINE',
			JustifyH = 'LEFT',
			JustifyV = 'TOP',
			AdjustX = -1,
			AdjustY = 8,
		},
		CountText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'RIGHT',
			JustifyV = 'BOTTOM',	
			AdjustX = 0,
			AdjustY = 0,
		},
	},
} )

KiwiPlates:RegisterSkin( 'Rounded.Small' , {
	Plate = { 
		left =0, top =-20, width  = 128, height =  32, 
		__HIDDEN_FIELDS = { width = true, height = true },
	},
	Mouseover = {
		disableHighlight = false,
		highlightColor = { 1,1,1,0.2 },
		filename = PATH .. 'small-high',
	},
	Target = {
		disableHighlight = false,	
		highlightColor = { 1,1,1,0.2 },
		filename  = PATH .. 'small-high',
	},	
	GlowBorder = {
		acceptColors = true,
		layer = "BACKGROUND",
		sublayer = -1,
		color = TRANSPARENT,
		filename = PATH .. 'small-border',
	},
	ClassificationIcon = {
		applyClassificationColor = true,
		left = -42, top = 0, width = 14, height = 14,
		display = { boss = true, elite = true, rare = true, normal = false, trivial = false, player = false },
		filename = PATH .. 'circle', __filename = 'ClassificationIcon',
	},
	HealthBar = {
		acceptColors = true,
		width = 92, height = 9,
		barTexture  = 'Gradient',
		Texture = { layer = "BACKGROUND", sublayer = 7, color = WHITE, filename = PATH .. 'small-bar' },
		__HIDDEN_FIELDS = { width = true, height = true },
	},
	NameText = {
		acceptColors = false,
		left = 0, top = 15,	width = 200, height = 20,
		fontName = FONT,
		fontColor = WHITE,
		fontSize =  11,
		fontFlags = 'OUTLINE',
		JustifyH = 'CENTER',
		JustifyV = 'BOTTOM',
	},
	TargetIcon = {
		left = 0, top = 16, width = 22, height = 22,
	},
	CastBar = {
		left = 0, top = -15, width = false, height = 12,		
		barTexture = 'Gradient',
		backColor = { 0.2, 0.2, 0.2, 1 },
		borderSize = 1,
		borderColor = BLACK,
		SpellIcon = {
			left = -81, top = 8, width = 26, height = 26,
			borderSize  = 1,
			borderColor = BLACK,
		},
		SpellText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'LEFT',
			JustifyV = 'MIDDLE',
			AdjustX = 0,
			AdjustY = 0,
		},
		TimeText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'RIGHT',
			JustifyV = 'MIDDLE',
			AdjustX = 0,
			AdjustY = 0,
		},
	},
	Auras = {
		left = -54, top = 27, 
		auraWidth = 22, auraHeight = 16, auraSpacing = 2,
		borderSize = 1,
		borderColor = BLACK,
		TimeText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  11,
			fontFlags = 'OUTLINE',
			JustifyH = 'LEFT',
			JustifyV = 'TOP',
			AdjustX = -1,
			AdjustY = 8,
		},
		CountText = {
			fontName = FONT,
			fontColor = WHITE,
			fontSize =  9,
			fontFlags = 'OUTLINE',
			JustifyH = 'RIGHT',
			JustifyV = 'BOTTOM',	
			AdjustX = 0,
			AdjustY = 0,
		},
	},
} )

KiwiPlates:RegisterWidgetTexture( 'ClassificationIcon', 'circle',  PATH .. 'circle' )
